@@ -10,7 +10,7 @@ use Bio::Gonzales::Util::File qw/open_on_demand/;
 use Bio::Gonzales::Util qw/flatten/;
 use base 'Exporter';
 our ( @EXPORT, @EXPORT_OK, %EXPORT_TAGS );
-our $VERSION = '0.0547_01'; # VERSION
+our $VERSION = 0.01_01;
 
 @EXPORT      = qw();
 %EXPORT_TAGS = ();
@@ -37,7 +37,7 @@ sub phylip_spew {
     }
   }
 
-  $fh->close unless ($fh_was_open);
+  close $fh unless ($fh_was_open);
 }
 
 sub _seq_phylip_spew {
@@ -70,7 +70,7 @@ sub phylip_slurp {
   elsif ( $mode =~ /^i(?:nter(?:leaved)?)?$/ ) { $seqs = _int_phylip_slurp( $fh, $relaxed ) }
   else                                         { croak "you have to supply a mode" }
 
-  $fh->close unless ($fh_was_open);
+  close $fh unless ($fh_was_open);
   return $seqs;
 }
 
