@@ -4,6 +4,7 @@ use warnings;
 use strict;
 use Carp;
 use Data::Dumper;
+use File::Slurp qw/slurp/;
 use List::MoreUtils qw/uniq/;
 use Bio::Gonzales::Util qw/flatten/;
 
@@ -16,102 +17,33 @@ use Bio::Gonzales::Util::File qw/open_on_demand slurpc/;
 
 use base 'Exporter';
 our ( @EXPORT, @EXPORT_OK, %EXPORT_TAGS );
-our $VERSION = '0.0547_01'; # VERSION
+our $VERSION = 0.01_01;
 
-@EXPORT      = qw(mslurp mspew lslurp miterate lspew dict_slurp dict_spew);
+@EXPORT      = qw(mslurp mspew lslurp miterate hslurp lspew dict_slurp dict_spew);
 %EXPORT_TAGS = ();
-@EXPORT_OK   = qw(lspew xlsx_slurp xlsx_spew);
+@EXPORT_OK   = qw(lspew xlsx_slurp xlsx_spew hspew);
 
 sub dict_slurp {
   my ( $src, $cc ) = @_;
-  croak "you have not specified key_idx and val_idx"
+  carp "you have not specified key_idx and val_idx, using 0 and 1"
     unless ( $cc && exists( $cc->{key_idx} ) && exists( $cc->{val_idx} ) );
 
-  $cc //= {};
-  my %c = (
-    sep              => qr/\t/,
-    header           => undef,
-    skip             => -1,
-    comment          => qr/^#/,
-    key_idx          => 0,
-    record_filter    => undef,
-    commented_header => undef,
-    concat_keys      => 1,
-    sort_keys      => 0,
-    %$cc
-  );
-
-  $c{header} //= $c{commented_header};
-
-  my $record_filter = $c{record_filter};
-
-  # concatenate keys to a big string
-  my @kidcs;
-  if ( $c{concat_keys} || !ref( $c{key_idx} ) ) {
-    @kidcs = ( $c{key_idx} );
-  } else {
-    # or treat them separately
-    @kidcs = @{ $c{key_idx} };
-  }
-
-  my $vidx = $c{val_idx};
+  my $kidx = $cc->{key_idx} // 0;
+  my $vidx = $cc->{val_idx} // 1;
   # make an array from it
+  $vidx = [$vidx]
+    unless ( ref $vidx );
 
-  my $uniq = $c{uniq} // $c{uniq_vals} // $c{unique} // 0;
-
-  my ( $fh, $fh_was_open ) = open_on_demand( $src, '<' );
-
-  my @header;
-  if ( $c{header} ) {
-    while ( my $raw_row = <$fh> ) {
-      if ( $c{comment} && $raw_row =~ /$c{comment}/ ) {
-        if ( $c{commented_header} ) {
-          $raw_row =~ s/$c{comment}//;
-        } else {
-          next;
-        }
-      }
-      $raw_row =~ s/\r\n/\n/;
-      chomp $raw_row;
-      @header = split /$c{sep}/, $raw_row;
-      last;
-    }
-  }
+  my @data = mslurp( $src, $cc );
 
+  my $matrix = shift @data;
   my %map;
-  my $lnum = 0;
-  while (<$fh>) {
-    next if ( $lnum++ <= $c{skip} );
-    next if ( $c{comment} && /$c{comment}/ );
-    s/\r\n/\n/;
-    chomp;
-    next if (/^\s*$/);
 
-    next if ( $record_filter && !$record_filter->($_) );
-
-    my @r = split /$c{sep}/;
-
-    for my $kidx (@kidcs) {
-
-      my @k = ( ref $kidx ? map { $_ // '' } @r[@$kidx] : $r[$kidx] );
-      @k = sort @k if ( $c{sort_keys} );
-      my $k = join( $;, @k ) // '';
-
-      if ( $uniq && !defined($vidx) ) {
-        $map{$k} = 1;
-      } elsif ( not defined $vidx ) {
-        $map{$k}++;
-      } elsif ($uniq) {
-        $map{$k} = ( ref $vidx ? [ @r[@$vidx] ] : $r[$vidx] );
-      } else {
-        $map{$k} //= [];
-        push @{ $map{$k} }, ( ref $vidx ? [ @r[@$vidx] ] : $r[$vidx] );
-      }
-    }
+  for my $l (@$matrix) {
+    $map{ $l->[$kidx] } //= [];
+    push @{ $map{ $l->[$kidx] } }, [ @{$l}[@$vidx] ];
   }
-
-  $fh->close unless ($fh_was_open);
-  return wantarray ? ( \%map, \@header ) : \%map;
+  return wantarray ? ( \%map, @data ) : \%map;
 }
 
 sub dict_spew {
@@ -140,67 +72,47 @@ sub mslurp {
 
   $cc //= {};
   my %c = (
-    sep              => qr/\t/,
-    header           => 0,
-    skip             => -1,
-    row_names        => 0,
-    comment          => qr/^#/,
-    commented_header => undef,
-    record_filter    => undef,
-    col_idx => undef,
+    sep       => qr/\t/,
+    header    => 0,
+    skip      => 0,
+    row_names => 0,
+    comment   => qr/^#/,
     %$cc
   );
 
-  my $record_filter = $c{record_filter};
-
-  my @col_idx;
-  @col_idx = @{$c{col_idx}} if($c{col_idx} && ref $c{col_idx} eq 'ARRAY');
   my @header;
   my @row_names;
 
   if ( $c{header} ) {
     while ( my $raw_row = <$fh> ) {
-      if ( $c{comment} && $raw_row =~ /$c{comment}/ ) {
-        if ( $c{commented_header} ) {
-          $raw_row =~ s/$c{comment}//;
-        } else {
-          next;
-        }
-      }
+      next if ( $c{comment} && $raw_row =~ /$c{comment}/ );
 
       $raw_row =~ s/\r\n/\n/;
       chomp $raw_row;
+      $raw_row =~ s/$c{pre}// if ( $c{pre} );
       @header = split /$c{sep}/, $raw_row;
       last;
     }
   }
 
-  my $lnum = 0;
   while (<$fh>) {
-    next if ( $lnum++ <= $c{skip} );
+    next if ( $. <= $c{skip} );
     next if ( $c{comment} && /$c{comment}/ );
     s/\r\n/\n/;
     chomp;
     next if (/^\s*$/);
 
-    next if ( $record_filter && !$record_filter->($_) );
-
     my @row = split /$c{sep}/;
-
     push @row_names, shift @row if ( $c{row_names} );
 
-    push @m, (@col_idx ? [ @row[@col_idx] ] : \@row);
+    push @m, \@row;
   }
-  $fh->close unless ($fh_was_open);
+  close $fh unless ($fh_was_open);
 
   #remove first empty element of a header if same number of elements as first matrix element.
   shift @header if ( $c{header} && @m > 0 && @{ $m[0] } == @header && !$header[0] );
 
-  if (wantarray) {
-    return ( \@m, ( @header ? \@header : undef ), ( @row_names ? \@row_names : undef ) );
-  } else {
-    return \@m;
-  }
+  return wantarray ? ( \@m, \@header, \@row_names ) : \@m;
 }
 
 sub miterate {
@@ -209,15 +121,12 @@ sub miterate {
 
   $cc //= {};
   my %c = (
-    sep           => qr/\t/,
-    skip          => 0,
-    comment       => qr/^#/,
-    record_filter => undef,
+    sep     => qr/\t/,
+    skip    => 0,
+    comment => qr/^#/,
     %$cc
   );
 
-  my $record_filter = $c{record_filter};
-
   return sub {
     while (<$fh>) {
 
@@ -226,14 +135,13 @@ sub miterate {
       next if ( $c{comment} && /$c{comment}/ );
       s/\r\n/\n/;
       chomp;
+      s/$c{pre}// if ( $c{pre} );
       next if (/^\s*$/);
-      next if ( $record_filter && !$record_filter->($_) );
-
       my @row = split /$c{sep}/;
       return \@row;
 
     }
-    $fh->close unless ($fh_was_open);
+    close $fh unless ($fh_was_open);
     return;
   };
 }
@@ -263,7 +171,7 @@ sub lspew {
   } else {
     confess "need a reference for the list argument";
   }
-  $fh->close unless ($fh_was_open);
+  close $fh unless ($fh_was_open);
 
   return;
 }
@@ -275,6 +183,29 @@ sub lslurp {
   return \@lines;
 }
 
+sub hspew { return lspew(@_) }
+
+sub hslurp {
+  my ( $file, $c ) = @_;
+
+  my $idx_col          = $c->{idx_col}    // 0;
+  my $allow_duplicates = $c->{duplicates} // 0;
+
+  my $lines = mslurp( $file, $c );
+  my %data;
+  for my $l (@$lines) {
+    my $key = splice @$l, $idx_col, 1;
+    if ($allow_duplicates) {
+      $data{$key} //= [];
+      push @{ $data{$key} }, $l;
+    } else {
+      confess if ( exists( $data{$key} ) );
+      $data{$key} = $l;
+    }
+  }
+  return \%data;
+}
+
 sub mspew {
   my ( $dest, $m, $c ) = @_;
 
@@ -292,7 +223,7 @@ sub mspew {
   # get the number of rows
   my $num_rows = scalar @$m;
   # find the longest column
-  my $num_cols = max - 1, map { defined $_ ? scalar @$_ : -1 } @$m;
+  my $num_cols = max map { defined $_ ? scalar @$_ : -1 } @$m;
 
   # if the header is longer, it defines the number of cols
   $num_cols = max $num_cols, scalar @$header if ( $header && @$header > 0 );
@@ -328,13 +259,13 @@ sub mspew {
     push @r, @{ $m->[$i] // [] } if ( $i < @$m );
     #fill the square if desired
     if ($square) {
-      my $missing = $num_cols - @r;
+      my $missing = $square - @r;
       push @r, (undef) x $missing;
     }
     #print row
     say $fh join $sep, @{ _quote( \@r, $quote_is_on, $na_value ) };
   }
-  $fh->close unless ($fh_was_open);
+  close $fh unless ($fh_was_open);
   return;
 }
 
@@ -351,7 +282,7 @@ sub _quote {
       ( my $str = $_ ) =~ s/"/""/g;
       qq{"$str"};
     }
-  } @$f;
+  } @{ $_[0] };
   return \@fields;
 }
 
@@ -392,13 +323,13 @@ sub xlsx_spew {
   my $worksheet = $workbook->add_worksheet();
   $worksheet->write_col( 'A1', \@table );
   $workbook->close;
-  $fh->close unless ($fh_was_open);
+  close $fh unless ($fh_was_open);
   return;
 }
 
 sub xlsx_slurp {
   my ( $src, $cc ) = @_;
-  #my @m;
+  my @m;
   #my ( $fh, $fh_was_open ) = open_on_demand( $src, '<' );
 
   eval "use Spreadsheet::XLSX; 1" or confess "could not load Spreadsheet::XLSX";
@@ -414,9 +345,7 @@ sub xlsx_slurp {
     my @m;
     my $cells = $sheet->{Cells};
     for my $r (@$cells) {
-      my @e;
-      for my $cell (@$r) { push @e, $cell->{Val}; }
-      push @m, \@e;
+      push @m, [ map { $_->{Val} } @$r ];
     }
     $ms{$sname} = \@m;
   }
@@ -444,41 +373,6 @@ Provides functions for common matrix/list IO.
 
 =over 4
 
-=item B<< dict_slurp($filename, \%options) >>
-
-  %options = (
-    sep     => qr/\t/,
-    header  => 0,
-    skip    => -1,
-    comment => qr/^#/,
-    key_idx => 0,
-    val_idx => undef,
-    uniq    => 0,
-    record_filter => undef,
-    concat_keys => 1,
-  );
-
-Setups:
-
-=over 4
-
-=item uniq = 1 && no val_idx => read in key_idx as hash and set values to 1
-
-=item uniq = 0 && no val_idx => read in key_idx as hash and set values to the count of keys
-
-=item uniq = 1 && val_idx    =>  read into ( key => [ @values ], ...)
-
-=item uniq = 0 && val_idx    =>  read into ( key => [ [ @values ], [ @more_values ] ], ...)
-
-=item concat_keys
-
-Concatenate the keys by C<< $; >>. If set to 0, key columns are taken in a
-serial fashion and are merged to one big column.
-
-=back
-
-If key_idx is an array, the keys columns are joined by C<$;> to build the hash key.
-
 =item B<< mspew($filename, \@matrix, \%options) >>
 
 =item B<< mspew($filehandle, \@matrix, \%options) >>
@@ -544,7 +438,6 @@ Further options with defaults:
         skip => 0, # skip the first N lines (without header)
         row_names => 0, # parse row names
         comment => qr/^#/ # the comment character
-        record_filter => undef # set a function to filter records
     );
     
 =item B<< lspew($fh_or_filename, $list, $config_options) >>
@@ -585,6 +478,45 @@ C<$config_options> is a hash ref. It can take the options:
     };
 
 
+=item B<< \%data = hslurp($file, \%config) >>
+
+Reads the context of a file, splits the input lines acording to mslurp rules
+and takes the first element as hash reference. The remaining part of the line
+is stored as array reference under the key (first element). The configuration
+options are the same as in mslurp.
+
+The default behaviour can be influenced by 
+
+  %config = (
+    idx_col => 0, # set a different index column as hash key, 0-based
+    has_duplicates => 0, # allow duplicate entries in the index column.
+    ...
+  );
+
+If duplicates are allowed, the result data structure has the form
+
+  %data = (
+    "key1" => [
+      [ elem_x, elem_y, elem_z ], 
+      [ elem_x, elem_y, elem_z ], 
+      ...
+    ],
+    "key2" => [
+      [ elem_x, elem_y, elem_z ], 
+      [ elem_x, elem_y, elem_z ], 
+      ...
+    ],
+  );
+
+If no duplicates are allowed, C<%data> has the structure
+
+  %data = (
+    "key1" => [ elem_x, elem_y, elem_z ], 
+    "key2" => [ elem_x, elem_y, elem_z ], 
+  );
+
+C<hslurp> dies with stacktrace if it hits a duplicate key in the index column.
+
 =back
 
 =head1 SEE ALSO
