@@ -10,7 +10,7 @@ use Bio::Phylo::IO qw/parse unparse/;
 
 use base 'Exporter';
 our ( @EXPORT, @EXPORT_OK, %EXPORT_TAGS );
-our $VERSION = '0.0547_01'; # VERSION
+our $VERSION = 0.01_01;
 
 @EXPORT      = qw();
 %EXPORT_TAGS = ();
@@ -68,7 +68,7 @@ sub consensus {
     #Bio::Phylo::Forest::Tree
     open my $outfh, '>', $outfile or die $!;
     print $outfh $consensus->to_newick( -nodelabels => 1 );
-    $outfh->close;
+    close $outfh;
 
 }
 
