@@ -12,13 +12,13 @@ use Bio::Gonzales::Util qw/flatten/;
 
 use base 'Exporter';
 our ( @EXPORT, @EXPORT_OK, %EXPORT_TAGS );
-our $VERSION = '0.0547_01'; # VERSION
+our $VERSION = 0.01_01;
 
 @EXPORT      = qw(faslurp faspew fasubseq faiterate);
 %EXPORT_TAGS = ();
 @EXPORT_OK   = qw(fahash);
 
-our $WIDTH = 80;
+our $WIDTH = 60;
 
 our $SEQ_FORMAT = 'all';
 
@@ -35,7 +35,8 @@ sub faslurp {
     push @fa, $entry;
   }
 
-  $fh->close unless ($fh_was_open);
+  close $fh
+    unless ($fh_was_open);
 
   return wantarray ? @fa : \@fa;
 }
@@ -83,7 +84,8 @@ sub fasubseq {
       push @fa, $entry unless (@$ranges);
     }
   }
-  $fh->close unless ($fh_was_open);
+  close $fh
+    unless ($fh_was_open);
 
   return wantarray ? @fa : \@fa;
 }
@@ -108,7 +110,7 @@ sub faiterate {
   return sub {
     my $entry = $fasta->next_seq;
     unless ( defined($entry) ) {
-      $fh->close unless ($fh_was_open);
+      close $fh unless ($fh_was_open);
 
       if ( my $src = shift @srcs ) {
         ( $fh, $fh_was_open ) = open_on_demand( $src, '<' );
@@ -148,7 +150,7 @@ sub faspew {
       confess "error";
     }
   }
-  $fh->close
+  close $fh
     unless ($fh_was_open);
   return;
 }
