@@ -8,11 +8,12 @@ use Carp;
 
 use 5.010;
 
+use Bio::Gonzales::Util::YAML qw/freeze_file/;
 use Bio::Gonzales::Seq::IO qw/faiterate faspew/;
 use Bio::Gonzales::Feat::IO::GFF3;
 use List::MoreUtils qw/all any none/;
 
-our $VERSION = '0.0547_01'; # VERSION
+our $VERSION = 0.01_01;
 
 has gff_file         => ( is => 'rw', required => 1 );
 has seq_files        => ( is => 'rw', default  => sub { [] } );
@@ -143,7 +144,7 @@ sub freeze_feats {
 sub freeze_ids {
     my ( $self, $file ) = @_;
 
-    yspew( $file, [ map { $_->id } $self->feats ] );
+    freeze_file( $file, [ map { $_->id } $self->feats ] );
 }
 
 sub freeze_seqs {
@@ -180,7 +181,7 @@ sub freeze_seqs {
     }
 
     #close filehandle
-    $faout_fh->close;
+    close $faout_fh;
 }
 
 1;
